/*
 * Copyright (C) 2018 Google LLC. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "config.h"

#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 101300) || (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 110000)
#import "SafeBrowsingContextProvider.h"

#if WK_API_ENABLED

#import "SafeBrowsingSPI.h"

@interface _WKSafeBrowsingTestServiceLookupResult : SSBServiceLookupResult
@property (nonatomic, readwrite, getter=isPhishing) BOOL phishing;
@end

@implementation _WKSafeBrowsingTestServiceLookupResult
@synthesize phishing=_phishing;
@end

@interface _WKSafeBrowsingTestLookupContext : SSBLookupContext
+ (_WKSafeBrowsingTestLookupContext *)sharedLookupContext;
- (void)lookUpURL:(NSURL *)URL completionHandler:(void (^)(SSBLookupResult *, NSError *))completionHandler;
@end

@implementation _WKSafeBrowsingTestLookupContext

+ (_WKSafeBrowsingTestLookupContext*) sharedLookupContext
{
    static _WKSafeBrowsingTestLookupContext* sharedTestLookupContext;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedTestLookupContext = [[self alloc] init];
    });

    return sharedTestLookupContext;
}

- (void)lookUpURL:(NSURL *)URL completionHandler:(void (^)(SSBLookupResult *, NSError *))completionHandler
{
    SSBLookupResult* lookupResult = [[SSBLookupResult alloc] init];
    _WKSafeBrowsingTestServiceLookupResult* result = [[_WKSafeBrowsingTestServiceLookupResult alloc] init];
    if ([[URL absoluteString] isEqualToString:@"sb://host/phishing.html"])
        [result setPhishing:YES];
    NSArray<SSBServiceLookupResult *> *serviceLookupResults = [NSArray arrayWithObject:[result autorelease]];
    [lookupResult setValue:serviceLookupResults forKey:@"serviceLookupResults"];
    completionHandler([lookupResult autorelease], nil);
}

@end

namespace WebKit {

bool SafeBrowsingContextProvider::s_useTestContext = false;

SSBLookupContext* SafeBrowsingContextProvider::sharedLookupContext()
{
    if (s_useTestContext)
        return [_WKSafeBrowsingTestLookupContext sharedLookupContext];
    return [SSBLookupContext sharedLookupContext];
}

} // namespace WebKit

#endif // WK_API_ENABLED

#endif
