/*
 * Copyright (C) 2018 Google LLC. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE INC. AND ITS CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL APPLE INC. OR ITS CONTRIBUTORS
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#pragma once

#import "WKFoundation.h"

#if WK_API_ENABLED

#import "WKNavigationDelegatePrivate.h"
#import "WebsitePoliciesData.h"
#import <WebCore/URL.h>
#import <WebCore/URLHash.h>
#import <wtf/HashSet.h>
#import <wtf/WeakPtr.h>

@class WKWebView;

namespace API {
class NavigationAction;
}

namespace WebKit {

class WebFramePolicyListenerProxy;

class SafeBrowsingController {
public:
    explicit SafeBrowsingController(WKWebView *);
    ~SafeBrowsingController();

    using NavigationPolicyDecider = WTF::Function<void(WTF::Function<void(WKNavigationActionPolicy, std::optional<WebsitePoliciesData>&&)>&&)>;

    void checkURL(Ref<API::NavigationAction>&&, NavigationPolicyDecider&&, Ref<WebFramePolicyListenerProxy>&&);

private:
    enum class SafeBrowsingResult { Safe, Unsafe };
    struct SafeBrowsingCheck {
        SafeBrowsingCheck(Ref<WebFramePolicyListenerProxy>&& listener, Ref<API::NavigationAction>&& navigationAction)
            : listener(WTFMove(listener))
            , navigationAction(WTFMove(navigationAction))
        {
        }

        std::optional<WKNavigationActionPolicy> navigationPolicyDecision;
        std::optional<WebsitePoliciesData> websitePoliciesData;
        std::optional<SafeBrowsingResult> safeBrowsingResult;
        Ref<WebFramePolicyListenerProxy> listener;
        Ref<API::NavigationAction> navigationAction;
    };

    void receivedNavigationPolicyDecision(SafeBrowsingCheck&, WKNavigationActionPolicy, std::optional<WebsitePoliciesData>&&);

    // Takes a |SafeBrowsingCheck*| rather than |SafeBrowsingCheck&| since the
    // check might have already been destroyed, e.g., if the embedder's navigation
    // policy response was to ignore the navigation.
    void receivedSafeBrowsingResult(SafeBrowsingCheck*, SafeBrowsingResult);

    void maybeFinishedCheck(SafeBrowsingCheck&);

    WKWebView *m_webView;
    HashSet<std::unique_ptr<SafeBrowsingCheck>> m_checks;
    HashSet<WebCore::URL> m_warningIgnoredUrls;
    WeakPtrFactory<SafeBrowsingController> m_weakPtrFactory;
    static const char* s_warningPageUrl;
};

} // namespace WebKit

#endif // WK_API_ENABLED
